﻿using System;
using System.Windows.Forms;//MessageBox.Show( "" );
using Plugin;
using NicoLibrary.NicoLiveAPI;

namespace ncvcommentmado
{
    //
    // region:    ncvcommentmado グローバルメンバ
    //
    public enum CommentType { Error = -1, Owner, BSP, Listener };
    
    public class _Main : IPlugin
    {
        //
        // region:    フィールド
        //

        IPluginHost host = null;
        CommentForm commentForm = null;
        const string PluginName = "コメント窓";
        const string PluginVersion = "1.0";
        const string PluginDescription = "独立したコメント窓を表示する。";

        //
        // region:    IPlugin メソッド
        //

        // method:  Run
        public void Run()
        {
            commentForm = initializeCommentForm();

            host.BroadcastConnected += host_BroadcastConnected;
            host.BroadcastDisConnected += host_BroadcastDisConnected;
        }

        // method:  AutoRun
        public void AutoRun()
        {
            Run();
        }

        //
        // region:    IPlugin プロパティ
        //

        public bool IsAutoRun       { get { return true; } }
        public string Description   { get { return PluginDescription; } }
        public string Name          { get { return PluginName; } }
        public string Version       { get { return PluginVersion; } }
        public IPluginHost Host
        {
            get { return host; }
            set { host = value; }
        }

        //
        // region:    イベントハンドラ（的なもの）
        //

        // eventhandler:    host_BroadcastConnected
        void host_BroadcastConnected( object sender, EventArgs e )
        {
            commentForm.HostBroadcastConnected();
        }

        // eventhandler:    host_BroadcastDisConnected
        void host_BroadcastDisConnected( object sender, EventArgs e )
        {
            commentForm.HostBroadcastDisConnected();
        }

        // eventhandler:    commentForm_FormClosed
        void commentForm_FormClosed( object sender, FormClosedEventArgs e )
        {
            uninitializeCommentForm( commentForm );

            host.BroadcastConnected -= host_BroadcastConnected;
            host.BroadcastDisConnected -= host_BroadcastDisConnected;
        }

        // eventhandler:    commentForm_PostCommand
        bool commentForm_PostCommand( 
            string comment, CommentType commentType, bool asAnonymous, 
            bool twitOrNot, string tweetconsolePath )
        {
            CommentType postedCommentType = postToNicoLive(
                comment, commentType, asAnonymous );

            if ( postedCommentType == CommentType.Error )
            {
                return false;
            }

            if ( twitOrNot && tweetconsolePath != "" )
            {
                postToTwitter( comment, postedCommentType, tweetconsolePath );
            }

            return true;
        }

        //
        // region:    サブルーチン
        //

        // subroutine:    initializeCommentForm
        CommentForm initializeCommentForm()
        {
            CommentForm form = new CommentForm();
            form.Text = PluginName;

            form.PostCommand += commentForm_PostCommand;
            form.FormClosed += commentForm_FormClosed;

            form.Show( host.MainForm );
            form.LoadSettings( host.DirectoryPathSettingFile );
            if ( host.IsConnected )
            {
                form.HostBroadcastConnected();
            }
            return form;
        }

        // subroutine:    uninitializeCommentForm
        void uninitializeCommentForm( CommentForm form )
        {
            form.SaveSettings( host.DirectoryPathSettingFile );
            form.PostCommand -= commentForm_PostCommand;
            form.FormClosed -= commentForm_FormClosed;
        }

        // subroutine:    postToNicoLive
        CommentType postToNicoLive(
            string comment, CommentType commentType, bool asAnonymous )
        {
            if ( commentType == CommentType.Owner ) // 運コメ
            {
                try // 放送主でなければ必ず失敗する
                {
                    if ( host.SendOwnerComment( 
                        asAnonymous, "", "", "", comment, "", "" ) )
                    {
                        return CommentType.Owner;
                    }
                }
                catch
                {
                    if ( host.SendComment( 
                        asAnonymous, "", "", "", comment ) )
                    {
                        return CommentType.Listener;
                    }
                }
            }
            else if ( commentType == CommentType.BSP ) // BSP
            {
                try // BSPでなければたぶん失敗する
                {
                    if ( host.SendPresscastComment ( "", comment, "" ) )
                    {
                        return CommentType.BSP;
                    }
                }
                catch
                {
                    if ( host.SendComment( 
                        asAnonymous, "", "", "", comment ) )
                    {
                        return CommentType.Listener;
                    }
                }
            }
            else // リスナー
            {
                if ( host.SendComment( asAnonymous, "", "", "", comment ) )
                {
                    return CommentType.Listener;
                }
            }

            return CommentType.Error;
        }

        // subroutine:    postToTwitter
        void postToTwitter( 
            string comment, CommentType commentType, string tweetconsolePath )
        {
            string args;
            NicoLivePlayerStatus playerStatus = host.GetPlayerStatus();
                
            args = "/t " + "\"" + "[ニコ生※] " + comment;
            if ( commentType == CommentType.Owner ) 
                { args = args + " [放送中] "; }
            else 
                { args = args + " [視聴中] "; }
            args = args + "http://live.nicovideo.jp/watch/lv"
                + playerStatus.LiveNum + "\"";

            System.Diagnostics.Process.Start(
                tweetconsolePath, args );
        }
    }
}
